<template>
  <view class="main">
    <view class="header-fixed">
      <uni-nav-bar background-color="transparent" :border="false" status-bar left-icon="left" title="班次设置" dark
        class="custom-nav-bar" @clickLeft="handleBack" />
    </view>
  </view>
  <view class="page-body">
    <view class="page-card">
      <view class="form-item" style="margin-top: 6rpx">
        <view class="form-item-label" data-required>班次名称</view>
        <input type="text" placeholder="请填写" class="form-item-input" v-model="shiftForm.name" />
      </view>
      <view class="form-item" style="margin-top: 48rpx">
        <view class="form-item-label" data-required>一次上下班</view>
        <PopupListPicker style="flex: 1;" title="上下班次数" :list="commutesNumberConfig"
          :activeIndex="shiftFormIndex.commutesNumberActiveIndex" @selectItem="handleSelectCommutesNumber">
          <view class="form-item-content">
            {{ commutesNumberConfig[shiftFormIndex.commutesNumberActiveIndex] }}
            <view class="iconfont icon-jinru-copy" />
          </view>
        </PopupListPicker>
      </view>
    </view>

    <view class="page-card">
      <view class="card-tips">
        {{ shiftFormIndex.commutesNumberActiveIndex === 0 ? '' : '第1次' }}上班时间设置
      </view>
      <ShiftAddForm @change="handleFormChange" prefix="number1" :form="shiftForm.number1" type="work" />
      <view class="card-tips">
        {{ shiftFormIndex.commutesNumberActiveIndex === 0 ? '' : '第1次' }}下班时间设置
      </view>
      <ShiftAddForm @change="handleFormChange" prefix="number1" :form="shiftForm.number1" type="off-work" />
    </view>

    <view class="page-card" v-if="shiftFormIndex.commutesNumberActiveIndex === 1">
      <view class="card-tips">
        第2次上班时间设置
      </view>
      <ShiftAddForm @change="handleFormChange" prefix="number2" :form="shiftForm.number2" type="work" />
      <view class="card-tips">
        第2次下班时间设置
      </view>
      <ShiftAddForm @change="handleFormChange" prefix="number2" :form="shiftForm.number2" type="off-work" />
    </view>

    <view class="page-card" v-if="shiftFormIndex.commutesNumberActiveIndex === 0">
      <view class="card-action-box">
        <view>
          <view>中途休息时间</view>
          <view class="card-tips" style="margin-top: 8rpx; font-size: 12px;">
            休息时间不计入工时
          </view>
        </view>
        <switch :checked="shiftForm.rest_time === 1" @change="handleRestTimeChange" />
      </view>
      <ShiftAddForm @change="handleFormChange" prefix="" :form="shiftForm" type="rest"
        :allowNextDay="shiftForm.number1.second_day_after === 1" />
    </view>
    <view class="overtime-title">加班起算时间</view>
    <view class="page-card">
      <view class="form-item">
        <view class="form-item-label" data-required>最后班次下班后多久开始计算</view>
        <picker mode="multiSelector" style="flex: 1;" :range="hourTimeRange" :value="shiftFormIndex.overtimeIndex"
          @change="handleOvertimeChange">
          <view class="form-item-content">
            {{ formatHourTime(shiftFormIndex.overtimeIndex) }}
            <view class="iconfont icon-jinru-copy" />
          </view>
        </picker>
      </view>
    </view>

    <view class="work-time-total-box">
      <view class="work-time-total-title">
        合计工作时长&nbsp;&nbsp;<text>{{ formatTotalTime(shiftForm.work_time) }}</text>
      </view>
      <view class="work-time-total-btn" @click="handleSave">保存</view>
    </view>
  </view>
</template>

<script setup lang="ts">
import PopupListPicker from "@/components/PopupListPicker/index.vue";
import ShiftAddForm from "./components/ShiftAddForm.vue";
import { hourTimeRange, formatHourTime, getInervalHour, getInervalTwoHour, getHour } from '@/utils/date';
import message from '@/utils/message';
import { attendanceScheduleShiftAddApi, attendanceScheduleShiftEditApi, attendanceScheduleShiftInfoApi } from '@/api/attendance';
import moment from "moment";

const props = withDefaults(defineProps<{
  id: string
}>(), {
  id: ''
});

const generateInitData = () => {
  return {
    first_day_after: 0, // 上班时间 0 => 当日, 1 => 次日
    work_hours: '09:00', // 上班时间
    late: 600, // 达到该时长即为迟到
    extreme_late: 1800, // 达到该时长即为严重迟到
    late_lack_card: 3600, // 达到该时长即为半天缺卡
    early_card: 1800, // 最多提前多久打卡

    second_day_after: 0, // 下班时间 0 => 当日, 1 => 次日
    off_hours: '18:00', // 下班时间
    early_leave: 600, // 提前多久打卡为早退
    early_lack_card: 1800, // 提前多久打卡为半天缺卡
    delay_card: 1800, // 最多延后多久打卡
    free_clock: 1 // 下班可免打卡
  };
}

const shiftForm = reactive<Record<string, any>>({
  color: "#1890ff", // 班次颜色

  name: '', // 班次名称
  number: 1, // 上下班次数
  work_time: '', // 工作总时间

  number1: generateInitData(),
  number2: generateInitData(),

  rest_time: 0, // 是否包含中途休息时间
  rest_start_after: 0, // 休息开始时间 0 => 当日, 1 => 次日
  rest_start: '12:00', // 休息开始时间
  rest_end_after: 0, // 休息结束时间 0 => 当日, 1 => 次日
  rest_end: '13:30', // 休息结束时间
  overtime: 0, // 多久下班后开始计算加班
});

const shiftFormIndex = ref({
  overtimeIndex: [0, 0],

  commutesNumberActiveIndex: 0, // 上下班次数列表索引
});

const commutesNumberConfig = [
  '一次上下班',
  '两次上下班'
];

// 选择上下班次数
const handleSelectCommutesNumber = (index: number) => {
  shiftFormIndex.value.commutesNumberActiveIndex = index;
  shiftForm.number = index === 0 ? 1 : 2;
  if (shiftForm.number === 1) {
    // 一次上下班
    shiftForm.number1.second_day_after = 0;
    shiftForm.number1.work_hours = "09:00";
    shiftForm.number1.off_hours = "18:00";
  } else {
    // 两次上下班
    shiftForm.number1.second_day_after = 0;
    shiftForm.number1.work_hours = "09:00";
    shiftForm.number1.off_hours = "12:00";
    shiftForm.number2.first_day_after = 0;
    shiftForm.number2.second_day_after = 0;
    shiftForm.number2.work_hours = "14:00";
    shiftForm.number2.off_hours = "18:00";
  }
  updateWorkTime(shiftForm);
};

const handleBack = () => {
  uni.navigateBack();
}

// 格式化合计工作时长
const formatTotalTime = (time: string) => {
  return time.replace('小时', ' 小时 ').replace('分钟', ' 分钟');
}

// 统一更新表单信息，如果是上下班时间设置，需要携带前缀
const handleFormChange = (prefix: string, value: Record<string, any>) => {
  if (prefix) {
    shiftForm[prefix] = {
      ...shiftForm[prefix],
      ...value
    };
  } else {
    Object.entries(value).forEach(([k, v]) => {
      shiftForm[k] = v;
    });
  }
  updateWorkTime(shiftForm);
}

// 选择是否包含中途休息时间
const handleRestTimeChange = (e: any) => {
  shiftForm.rest_time = e.detail.value ? 1 : 0;
  updateWorkTime(shiftForm);
}

// 选择加班起算时间
const handleOvertimeChange = (e: any) => {
  shiftFormIndex.value.overtimeIndex = e.detail.value;
  shiftForm.overtime = e.detail.value[0] * 3600 + e.detail.value[1] * 60;
}

// 保存
const handleSave = async () => {
  uni.showLoading({
    title: '保存中...',
    mask: true
  });

  const fn = props.id ? () => attendanceScheduleShiftEditApi(props.id, shiftForm) : () => attendanceScheduleShiftAddApi(shiftForm);

  try {
    const res = await fn();
    uni.hideLoading();
    message.success(res.message, 'success');
    setTimeout(() => {
      uni.navigateBack();
    }, 1000);
  } catch (err) {
    uni.hideLoading();
    message.error(err.message, 'error');
  }
}

// 更新合计工作时长
function updateWorkTime(newVal: any) {
  let duration1: any[] = [];
  if (newVal.number == 1 && newVal.rest_time == 1) {
    if (newVal.number1.second_day_after == 0) {
      if (newVal.number1.work_hours > newVal.rest_start) {
        return message.error('中途休息开始时间要大于上班时间', 'error')
      }
      if (newVal.number1.off_hours < newVal.rest_end) {
        return message.error('中途休息结束时间要小于下班时间', 'error')
      }
    } else {
      if (newVal.rest_start_after == 0) {
        if (newVal.number1.work_hours > newVal.rest_start) {
          return message.error('中途休息开始时间要大于上班时间', 'error')
        }
      }

      if (newVal.rest_end_after == 1) {
        if (newVal.number1.off_hours < newVal.rest_end) {
          return message.error('中途休息结束时间要小于下班时间', 'error')
        }
      }
    }
    if (!newVal.rest_time) {
      if (newVal.number1.work_hours && newVal.number1.off_hours) {
        if (newVal.number1.second_day_after == 0) {
          duration1 = getInervalHour(newVal.number1.work_hours, newVal.number1.off_hours)
        } else {
          duration1 = getInervalHour(newVal.number1.work_hours, newVal.number1.off_hours)
          duration1[0] = parseInt((duration1[0] + 24) + "")
        }
      }
    } else {
      if (newVal.rest_start && newVal.rest_end) {
        if (newVal.number1.second_day_after == 0) {
          newVal.rest_start_after = 0
          newVal.rest_end_after = 0

          duration1 = getInervalTwoHour(
            newVal.number1.work_hours,
            newVal.number1.off_hours,
            newVal.rest_start,
            newVal.rest_end
          )
        } else {
          if (newVal.rest_start_after == 1) {
            newVal.rest_end_after = 1
          }

          duration1 = getInervalTwoHour(
            newVal.number1.work_hours,
            newVal.number1.off_hours,
            newVal.rest_start,
            newVal.rest_end
          )
          if (newVal.rest_start_after == 0 && newVal.rest_end_after == 1) {
            duration1[0] = parseInt((duration1[0] + 24 - 24) + "")
          } else {
            duration1[0] = parseInt((duration1[0] + 24) + "")
          }
        }
      }
    }
  } else {
    if (newVal.number1.second_day_after == 1) {
      newVal.number2.first_day_after = 1
    }
    if (newVal.number2.first_day_after == 1) {
      newVal.number2.second_day_after = 1
    }
    if (newVal.number1.second_day_after == 0) {
      const time1 = new Date(`1970-01-01 ${newVal.number1.off_hours}`).getTime();
      const time2 = new Date(`1970-01-01 ${newVal.number1.work_hours}`).getTime();
      if (time1 < time2) {
        return message.error('下班时间要大于上班时间', 'error');
      }
    }
    if (newVal.number == 1) {
      duration1 = getHour(newVal.number1.work_hours, newVal.number1.off_hours, 0, 0)
    } else {
      duration1 = getHour(
        newVal.number1.work_hours,
        newVal.number1.off_hours,
        newVal.number2.work_hours,
        newVal.number2.off_hours
      )
    }

    if (newVal.number1.second_day_after == 1) {
      duration1[0] = parseInt((duration1[0] + 24) + "")
    }
    if (newVal.number2.first_day_after == 0 && newVal.number2.second_day_after == 1) {
      duration1[0] = parseInt((duration1[0] + 24) + "")
    }
  }

  shiftForm.work_time = duration1[0] + '小时' + duration1[1] + '分钟'
}

updateWorkTime(shiftForm);

// 获取班次信息
const getShiftInfo = () => {
  attendanceScheduleShiftInfoApi(props.id).then(({ data }: any) => {
    for (const key in shiftForm) {
      if (data[key] && !Array.isArray(data[key])) {
        if (key === 'number1' || key === 'number2') {
          for (const subKey in shiftForm[key]) {
            if (data[key][subKey]) shiftForm[key][subKey] = data[key][subKey];
          }
        } else {
          shiftForm[key] = data[key];
        }
      }
    }

    const duration = moment.duration(shiftForm.overtime, 'seconds');

    // 提取小时和分钟
    const hours = duration.hours();
    const minutes = duration.minutes();

    shiftFormIndex.value.overtimeIndex = [hours, minutes];
  });
}


props.id && getShiftInfo();


</script>

<style scoped lang="scss">
.main {
  padding-top: calc(44px + var(--status-bar-height));
  padding-bottom: 24rpx;
}

.header-fixed {
  position: fixed;
  top: 0;
  left: 0;
  right: 0;
  z-index: 1;
}

.custom-nav-bar {
  background: linear-gradient(90deg, #459FFF 0%, #388AEF 100%, #3384E7 100%);
}

.page-body {
  padding: 20rpx;
  padding-bottom: 260rpx;
  font-weight: 400;
  font-size: 15px;
  color: #303133;
  line-height: 42rpx;
}

.page-card {
  background-color: #fff;
  border-radius: 12rpx;
  padding: 24rpx;

  &+& {
    margin-top: 20rpx;
  }
}

.form-item {
  display: flex;
  align-items: center;

  .form-item-label {
    &[data-required] {
      &::after {
        content: '*';
        color: #FF2529;
        margin-left: 8rpx;
        vertical-align: middle;
      }
    }
  }

  .form-item-content,
  .form-item-input {
    flex: 1;
    text-align: right;
    margin-left: 20rpx;
  }

  .form-item-content {
    display: flex;
    align-items: center;
    justify-content: flex-end;
    color: #606266;

    .iconfont {
      font-size: 12px;
      color: #C0C4CC;
      margin-left: 14rpx;
    }
  }
}

:deep(.input-placeholder) {
  color: #C0C4CC;
}

.card-tips {
  font-size: 26rpx;
  color: #909399;
  line-height: 36rpx;
  margin-bottom: 20rpx;

  &:not(:first-child) {
    margin-top: 34rpx;
  }
}

.overtime-title {
  padding: 30rpx 0 20rpx 24rpx;
}

.card-action-box {
  display: flex;
  justify-content: space-between;
  align-items: center;

  switch {
    transform: scale(0.75);
  }
}

.work-time-total-box {
  position: fixed;
  bottom: 0;
  left: 0;
  right: 0;
  background-color: #fff;
  padding: 24rpx 20rpx;
}

.work-time-total-title {
  margin-bottom: 24rpx;
  font-size: 26rpx;
  color: #909399;

  text {
    color: #303133;
  }
}

.work-time-total-btn {
  height: 86rpx;
  background: #1890FF;
  border-radius: 12rpx;
  color: #fff;
  display: flex;
  align-items: center;
  justify-content: center;
}
</style>
